import { format } from 'prettier'
import { snapshot } from 'valtio'

import { FunctionFragment, FunctionFragmentOptions } from '../fragments'
import { mergeUrls, removeLinksFromText } from '../helpers'
import { appState } from '../state'
import { Signature } from '../types'

/**
 * Creates a page template for a function.
 *
 * @param signature - Function signature
 * @param originalDocument - Auto-generated JSON file
 * @param slug - Slug to use for generating documentation links
 * @param functionFragmentOptions - Options for the function fragment
 * @returns Prettified function page template
 */
export const FunctionTemplate = (
  { id, name, comment, signatures, sources }: Signature,
  originalDocument?: Array<Signature>,
  slug?: string,
  functionFragmentOptions?: FunctionFragmentOptions
) => {
  const alias = comment?.tags?.find(({ tag }) => tag === 'alias')?.text.replace(/\n/g, '')
  const allChildrenDeprecated = signatures?.every((signature) =>
    signature.comment?.tags?.some(({ tag }) => tag === 'deprecated')
  )
  const { baseEditUrl, contentReferences } = snapshot(appState)
  const isComponent = contentReferences.get(id) === 'Component'
  const source = sources?.[0]

  const header = `---
# ⚠️ AUTO-GENERATED CONTENT. DO NOT EDIT THIS FILE DIRECTLY! ⚠️
title: ${isComponent ? `<${name} />` : `${name}()`}
sidebar_label: ${isComponent ? `<${alias || name} />` : `${alias || name}()`}
${slug ? `slug: ${slug}` : ``}
${allChildrenDeprecated ? 'sidebar_class_name: deprecated' : ''}
${
  signatures && signatures.length > 0
    ? `description: ${
        removeLinksFromText(signatures[0].comment?.shortText?.replace(/\n/gi, ' ')) ||
        'No description provided.'
      }`
    : 'description: No description provided.'
}
${
  baseEditUrl && source
    ? `custom_edit_url: ${mergeUrls(baseEditUrl, `${source.fileName}#L${source.line}`)}`
    : ''
}
---`.replace(/\n\n/gi, '\n')

  return format(
    `${header}

${signatures && signatures.length > 1 ? `# \`${name}()\`` : ``}

${
  signatures
    ? signatures
        .map((signature, index) =>
          FunctionFragment(signature, originalDocument, {
            numberOfOverloads: signatures.length,
            index,
            ...functionFragmentOptions
          })
        )
        .join('\n\n')
    : ''
}
`,
    { parser: 'markdown', semi: false, singleQuote: true, trailingComma: 'none' }
  )
}

export default FunctionTemplate
